<?php
/**
 * @version       1.2.0
 * @package       Joomla
 * @subpackage    Event Booking
 * @author        Denis Moskvin
 * @copyright     Copyright (C) 2016 PayAnyWay
 * @license       GNU/GPL, see LICENSE.php
 */
// no direct access
defined('_JEXEC') or die();

class os_payanyway extends os_payment
{

	const paw_kassa_VAT0     = '1104';  // НДС 0%
	const paw_kassa_VAT10    = '1103';  // НДС 10%
	const paw_kassa_VAT18    = '1102';  // НДС 18%
	const paw_kassa_VATNOVAT = '1105';  // НДС не облагается
	const paw_kassa_VATWR10  = '1107';  // НДС с рассч. ставкой 10%
	const paw_kassa_VATWR18  = '1106';  // НДС с рассч. ставкой 18%

	private static function monetaPayURLResponse($mnt_id, $mnt_transaction_id, $mnt_data_integrity_code, $success = false,
												 $repeatRequest = false, $echo = true, $die = true, $kassa_inventory = null, $kassa_customer = null, $kassa_delivery = null)
	{
		if ($success === true)
			$resultCode = '200';
		elseif ($repeatRequest === true)
			$resultCode = '402';
		else
			$resultCode = '500';
		$mnt_signature = md5($resultCode.$mnt_id.$mnt_transaction_id.$mnt_data_integrity_code);
		$response = '<?xml version="1.0" encoding="UTF-8" ?>'."\n";
		$response .= "<MNT_RESPONSE>\n";
		$response .= "<MNT_ID>{$mnt_id}</MNT_ID>\n";
		$response .= "<MNT_TRANSACTION_ID>{$mnt_transaction_id}</MNT_TRANSACTION_ID>\n";
		$response .= "<MNT_RESULT_CODE>{$resultCode}</MNT_RESULT_CODE>\n";
		$response .= "<MNT_SIGNATURE>{$mnt_signature}</MNT_SIGNATURE>\n";
		if (!empty($kassa_inventory) || !empty($kassa_customer) || !empty($kassa_delivery))
		{
			$response .= "<MNT_ATTRIBUTES>\n";
			foreach (array('INVENTORY' => $kassa_inventory, 'CUSTOMER' => $kassa_customer, 'DELIVERY' => $kassa_delivery) as $k => $v)
				if (!empty($v))
					$response .= "<ATTRIBUTE><KEY>{$k}</KEY><VALUE>{$v}</VALUE></ATTRIBUTE>\n";
			$response .= "</MNT_ATTRIBUTES>\n";
		}
		$response .= "</MNT_RESPONSE>\n";
		if ($echo === true)
		{
			header("Content-type: application/xml");
			echo $response;
		}
		else
			return $response;
		if ($die === true)
			die;
		return '';
	}

	/**
	 * payanyway mode
	 *
	 * @var boolean live mode : true, test mode : false
	 */
	var $_mode = 0;

	/**
	 * payanyway url
	 *
	 * @var string
	 */
	var $_url = null;

	/**
	 * Array of params will be posted to server
	 *
	 * @var string
	 */
	var $_params = array();

	/**
	 * Array of post params
	 *
	 * @var array
	 */
	var $_post_params = array();

	/**
	 * Array containing data posted from payanyway to our server
	 *
	 * @var array
	 */
	var $_data = array();

	/**
	 * Constructor functions, init some parameter
	 *
	 * @param $params
	 * @internal param object $config
	 */
	function os_payanyway($params)
	{
		parent::setName('os_payanyway');
		parent::os_payment();
		parent::setCreditCard(false);
		parent::setCardType(false);
		parent::setCardCvv(false);
		parent::setCardHolderName(false);

		$this->_mode = $params->get('payanyway_mode');
		if ($this->_mode)
		{
			$this->_url = 'https://demo.moneta.ru/assistant.htm';
		} else
		{
			$this->_url = 'https://www.payanyway.ru/assistant.htm';
		}

//		$this->_url = 'http://localhost:8080/assistant.htm';

		$currency = $params->get('payanyway_currency', 'RUB');
		if ($currency == 'RUR')
		{
			$currency = 'RUB';
		}

		$this->setParam('currency_code', $currency);
		$this->setParam('business', $params->get('account_id'));

		// other params
		$this->setParam('rm', 2);
		$this->setParam('cmd', '_xclick');
		$this->setParam('no_shipping', 1);
		$this->setParam('no_note', 1);
		$this->setParam('lc', 'RU');
		$this->setParam('charset', 'utf-8');
		$this->setParam('tax', 0);

		// payanyway specific params
		$this->setParam('account_id', $params->get('account_id'));
		$this->setParam('account_code', $params->get('account_code'));
		$this->setParam('payanyway_test', $params->get('payanyway_test'));

	}

	/**
	 * Set param value
	 *
	 * @param string $name
	 * @param string $val
	 */
	function setParam($name, $val)
	{
		$this->_params[$name] = $val;
	}

	/**
	 * Setup payment parameter
	 *
	 * @param array $params
	 */
	function setParams($params)
	{
		foreach ($params as $key => $value)
		{
			$this->_params[$key] = $value;
		}
	}

	/**
	 * Set Post param value
	 *
	 * @param string $name
	 * @param string $val
	 */
	function setPostParam($name, $val)
	{
		$this->_post_params[$name] = $val;
	}

	/**
	 * Setup Post payment parameter
	 *
	 * @param array $params
	 */
	function setPostParams($params)
	{
		foreach ($params as $key => $value)
		{
			$this->_post_params[$key] = $value;
		}
	}

	/**
	 * Process Payment
	 *
	 * @param object $row
	 * @param $data
	 * @internal param array $params
	 */
	function processPayment($row, $data)
	{
		$Itemid = JRequest::getInt('Itemid', 0);
		$siteUrl = JUri::base();

		$itemName = JText::_('EB_EVENT_REGISTRATION');
		$itemName = str_replace('[EVENT_TITLE]', $data['event_title'], $itemName);

		$description = "Address: " . EventbookingHelper::getCountryCode($row->country) . "{$row->city} {$row->state} {$row->zip} {$row->address} {$row->address2}, Client: {$row->first_name} {$row->last_name} {$row->email}, Order: {$itemName} | {$Itemid} | {$row->id} | {$row->event_id}";
		$amount = number_format($data['amount'], 2, '.', '');

		$transactionId = $Itemid . '_' . $row->event_id . '_' . $row->id;

		$this->setPostParam('MNT_ID', $this->_params['account_id']);
		$this->setPostParam('MNT_AMOUNT', $amount);
		$this->setPostParam('MNT_CURRENCY_CODE', $this->_params['currency_code']);
		$this->setPostParam('MNT_DESCRIPTION', $description);
		$this->setPostParam('MNT_TRANSACTION_ID', $transactionId);

		$test_mode = $this->_params['payanyway_test'];
		if (!$test_mode)
		{
			$test_mode = '0';
		}

		$signature = md5($this->_params['account_id'] . $transactionId . $amount . $this->_params['currency_code'] . $test_mode . $this->_params['account_code']);
		$this->setPostParam('MNT_SIGNATURE', $signature);

		$this->setPostParam('MNT_SUCCESS_URL', $siteUrl . 'index.php?option=com_eventbooking&view=complete&Itemid=' . $Itemid);
		$this->setPostParam('MNT_FAIL_URL', $siteUrl . 'index.php?option=com_eventbooking&task=cancel&id=' . $row->id . '&Itemid=' . $Itemid);

		// Pay URL:
		// $siteUrl . 'index.php?option=com_eventbooking&task=payment_confirm&payment_method=os_payanyway'

		$this->submitPost();
	}

	/**
	 * Submit post to payanyway server
	 *
	 */
	function submitPost()
	{
		?>
		<div class="contentheading">Process payment ... Please wait ...</div>
		<form method="post" action="<?php echo $this->_url; ?>" name="jd_form" id="jd_form">
			<?php
			foreach ($this->_post_params as $key => $val)
			{
				echo '<input type="hidden" name="' . $key . '" value="' . $val . '" />';
				echo "\n";
			}
			?>
			<script type="text/javascript">
				function redirect()
				{
					document.jd_form.submit();
				}
				setTimeout('redirect()', 1000);
			</script>
		</form>
		<?php
	}


	function getRequestVar($name)
	{
		$value = null;
		if (isset($_POST[$name]))
		{
			$value = $_POST[$name];
		} else if (isset($_GET[$name]))
		{
			$value = $_GET[$name];
		}
		return $value;
	}

	/**
	 * Process payment
	 *
	 */
	function verifyPayment()
	{

		$errorHandlerFileName = date("Ymd") . ".txt";
		$errorHandlerMessage = "POST: " . print_r($_POST, true) . " GET: " . print_r($_GET, true);

		$fp = fopen($errorHandlerFileName, "a");
		if ($fp)
		{
			flock($fp, LOCK_EX);
			fwrite($fp, $errorHandlerMessage);
			flock($fp, LOCK_UN);
			fclose($fp);
		}

		$config = EventbookingHelper::getConfig();

		// Request params
		$MNT_ID = $this->getRequestVar('MNT_ID');
		$MNT_TRANSACTION_ID = $this->getRequestVar('MNT_TRANSACTION_ID');
		$MNT_OPERATION_ID = $this->getRequestVar('MNT_OPERATION_ID');
		$MNT_AMOUNT = $this->getRequestVar('MNT_AMOUNT');
		$MNT_CURRENCY_CODE = $this->getRequestVar('MNT_CURRENCY_CODE');
		$MNT_SUBSCRIBER_ID = $this->getRequestVar('MNT_SUBSCRIBER_ID');
		$MNT_TEST_MODE = $this->getRequestVar('MNT_TEST_MODE');
		if (!$MNT_TEST_MODE)
		{
			$MNT_TEST_MODE = '0';
		}

		$MNT_SIGNATURE = $this->getRequestVar('MNT_SIGNATURE');
		// check signature
		$signature = md5($MNT_ID . $MNT_TRANSACTION_ID . $MNT_OPERATION_ID . $MNT_AMOUNT . $MNT_CURRENCY_CODE . $MNT_SUBSCRIBER_ID . $MNT_TEST_MODE . $this->_params['account_code']);
		if ($MNT_SIGNATURE != $signature)
		{
			echo "FAIL";
			exit;
		}

		$id = $MNT_TRANSACTION_ID;
		if (strpos($id, '_') !== false)
		{
			$transactionIdArray = explode('_', $id);
			if (isset($transactionIdArray[2]))
			{
				$id = $transactionIdArray[2];
			}
		}

		$row = JTable::getInstance('EventBooking', 'Registrant');
		$row->load($id);
		if (!$row->id)
		{
			echo "FAIL";
			exit;
		}
		if ($row->published)
		{
			echo "FAIL";
			exit;
		}

		$evt = JTable::getInstance('EventBooking', 'Event');
		$evt->load($row->event_id);
		$vatTag = self::paw_kassa_VATNOVAT;
		if ($row->tax_amount > 0)
		{
			$taxRate = round($row->total_amount / $row->tax_amount);
			if ($taxRate == 10)
				$vatTag = self::paw_kassa_VAT10;
			if ($taxRate == 18)
				$vatTag = self::paw_kassa_VAT18;
		}
		$kassa_inventory = json_encode(array(array(
			'name' => trim(preg_replace("/&?[a-z0-9]+;/i", "", htmlspecialchars($evt->title.' ('.$evt->event_date.' - '.$evt->event_end_date.')'))),
			'price' => $row->amount,
			'quantity' => '1',
			'vatTag' => $vatTag
		)));

		// save payment transaction
		$row->payment_date = gmdate('Y-m-d H:i:s');
		$row->published = true;
		$row->checked_in = 1;
		$row->store();
		if ($row->is_group_billing)
		{
			EventbookingHelper::updateGroupRegistrationRecord($row->id);
		}
		EventbookingHelper::sendEmails($row, $config);
		JPluginHelper::importPlugin('eventbooking');
		$dispatcher = JDispatcher::getInstance();
		$dispatcher->trigger('onAfterPaymentSuccess', array($row));

		self::monetaPayURLResponse($MNT_ID, $MNT_TRANSACTION_ID, $this->_params['account_code'], true, false, true, false, $kassa_inventory, $row->email);
		exit;
	}
}
